from naas_drivers.driver import InDriver
import pandas as pd
import datetime as dt


class Yahoofinance(InDriver):
    def get(
        self,
        stock_companies,
        date_from=-36,
        date_to="today",
        interval="1d",
        moving_averages=[],
    ):
        """ generate financial data """
        if isinstance(stock_companies, str):
            stock_companies = [stock_companies]
        if isinstance(date_from, int) and date_from < 0:
            date_from = dt.datetime.today() + dt.timedelta(days=date_from)
        else:
            error_text = f"date_from ({date_from}) cannot be positive"
            self.print_error(error_text)
            return None
        if isinstance(date_to, int) and date_to > 0:
            date_to = dt.datetime.today() + dt.timedelta(days=date_to)
        if isinstance(date_to, str) and date_to == "today":
            date_to = dt.datetime.today()
        else:
            error_text = f"date_to ({date_to}) cannot be negative"
            self.print_error(error_text)
            return None
        stocks = None
        period1 = date_from.strftime("%s")
        period2 = date_to.strftime("%s")
        for company in stock_companies:
            url = (
                f"https://query1.finance.yahoo.com/v7/finance/download/"
                f"{company}?period1={period1}&period2={period2}&interval={interval}&events=history"
            )
            stock = pd.read_csv(url)
            stock["Company"] = company
            for moving_average in moving_averages:
                stock[f"MA{moving_average}"] = stock.Close.rolling(
                    moving_average
                ).mean()
            if stocks is None:
                stocks = stock
            else:
                stocks = stocks.append(stock)
            print("getted data for", company, url)
        stocks["Date"] = pd.to_datetime(stocks["Date"], format="%Y-%m-%d")
        return stocks



from naas_drivers.driver import InDriver, OutDriver
from notion.client import NotionClient
import pandas as pd
import requests
import os


class Notion(InDriver, OutDriver):
    def __init__(self):
        self.auth_proxy = os.getenv("NAAS_AUTH_PROXY")

    def connect(
        self,
        token: str = None,
        email: str = None,
        password: str = None,
    ):
        if token:
            self.client = NotionClient(token)
        elif email and password:
            self.client = NotionClient(token_v2=self.__token_v2(email, password))
        else:
            self.print_error("You should provide, token or email/pasword")
        self.connected = True
        return self

    def __token_v2(
        self,
        email: str,
        password: str,
    ):
        url = f"https://www.notion.so/login&filter=token_v2&email={email}&password={password}"
        cookie_response = requests.get(f"{self.auth_proxy}/token?url={url}")
        return cookie_response.json()["cookies"][0]["value"]

    def get(self, url: str):
        self.check_connect()
        page = self.client.get_block(url)
        return page

    def get_collection(
        self,
        url: str,
        raw: bool = False,
    ):
        self.check_connect()
        cv = self.client.get_collection_view(url)
        if raw:
            return cv
        else:
            data = [
                block_row.get_all_properties() for block_row in cv.collection.get_rows()
            ]
            return pd.DataFrame(data)



from naas_drivers.driver import ConnectDriver
import requests


class CRUD(ConnectDriver):
    req_headers = {"Accept": "application/json", "Content-Type": "application/json"}
    base_public_url = None
    endpoint = None
    key = None
    raise_error = False

    def __init__(
        self,
        base_url,
        endpoint,
        auth,
    ):
        self.base_public_url = base_url
        self.endpoint = endpoint
        self.key = auth

    def get(self, uid):
        self.check_connect()
        req = requests.get(
            url=f"{self.base_public_url}/{self.endpoint}/{uid}",
            headers=self.req_headers,
            auth=self.key,
            allow_redirects=False,
        )
        if self.raise_error:
            req.raise_for_status()
        return req

    def send(self, data):
        self.check_connect()
        req = requests.post(
            url=f"{self.base_public_url}/{self.endpoint}",
            auth=self.key,
            headers=self.req_headers,
            json=data,
            allow_redirects=False,
        )
        if self.raise_error:
            req.raise_for_status()
        return req

    def update(self, data):
        self.check_connect()
        _id = data.get("_id")
        req = requests.put(
            url=f"{self.base_public_url}/{self.endpoint}/{_id}",
            auth=self.key,
            headers=self.req_headers,
            json=data,
            allow_redirects=False,
        )
        if self.raise_error:
            req.raise_for_status()
        return req

    def delete(self, data):
        self.check_connect()
        _id = data.get("_id")
        req = requests.delete(
            url=f"{self.base_public_url}/{self.endpoint}/{_id}",
            auth=self.key,
            headers=self.req_headers,
            allow_redirects=False,
        )
        if self.raise_error:
            req.raise_for_status()
        return req
